"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = require("node:path");
const extract_styles_1 = require("../postcss/extract-styles");
const append_variables_1 = require("./transforms/append-variables");
const append_import_1 = require("./transforms/append-import");
const visitor_1 = require("./visitor");
const allowed_paths_1 = require("./utils/allowed-paths");
const get_tailwind_config_1 = require("./utils/get-tailwind-config");
const serialize_styles_1 = require("../utils/serialize-styles");
function rootVisitor(options, cwd) {
    const errors = [];
    const tailwindConfigPath = (0, node_path_1.resolve)(cwd, options.tailwindConfigPath || "./tailwind.config.js");
    const tailwindConfig = (0, get_tailwind_config_1.getTailwindConfig)(tailwindConfigPath, {
        ...options,
        onError(error) {
            errors.push(error);
        },
    });
    const { allowModuleTransform, allowRelativeModules } = (0, allowed_paths_1.getAllowedOptions)(tailwindConfig, options);
    return {
        visitor: {
            Program: {
                enter(path, state) {
                    if (!(0, allowed_paths_1.isAllowedProgramPath)({
                        path: state.filename,
                        allowRelativeModules,
                        cwd,
                    })) {
                        return;
                    }
                    let canCompile = true;
                    let canTransform = true;
                    switch (state.opts.mode) {
                        case "compileOnly": {
                            canTransform = false;
                            break;
                        }
                        case "transformOnly": {
                            canCompile = false;
                            break;
                        }
                    }
                    const visitorState = {
                        cwd,
                        rem: 16,
                        platform: "native",
                        hmr: true,
                        mode: "compileAndTransform",
                        didTransform: false,
                        blockModuleTransform: [],
                        hasStyledComponentImport: false,
                        canCompile,
                        canTransform,
                        ...state,
                        ...state.opts,
                        allowModuleTransform,
                        allowRelativeModules,
                        blockList: new Set(),
                        hasProvider: false,
                        hasStyleSheetImport: false,
                        tailwindConfig,
                        tailwindConfigPath,
                    };
                    // Traverse the file
                    path.traverse(visitor_1.visitor, visitorState);
                    const { filename, hasStyleSheetImport, didTransform, hasProvider, hasStyledComponentImport, hmr, } = visitorState;
                    /*
                     * If we are not hmr, we only care if this file has a provider
                     */
                    if (!hmr && !hasProvider) {
                        return;
                    }
                    const extractStylesOptions = hmr
                        ? {
                            ...tailwindConfig,
                            content: [filename],
                            // If the file doesn't have any Tailwind styles, it will print a warning
                            // We force an empty style to prevent this
                            safelist: ["babel-empty"],
                            serializer: serialize_styles_1.babelStyleSerializer,
                        }
                        : {
                            ...tailwindConfig,
                            serializer: serialize_styles_1.babelStyleSerializer,
                        };
                    const bodyNode = path.node.body;
                    if (didTransform && !hasStyledComponentImport) {
                        (0, append_import_1.prependImports)(bodyNode, ["StyledComponent"], "tailwindcss-react-native");
                    }
                    const { output } = (0, extract_styles_1.extractStyles)(extractStylesOptions);
                    if (!output.hasStyles)
                        return;
                    (0, append_variables_1.appendVariables)(bodyNode, output);
                    const imports = [];
                    if (!hasStyleSheetImport) {
                        imports.push(["RNStyleSheet", "StyleSheet"]);
                    }
                    if (output.hasPlatform) {
                        imports.push(["RNPlatform", "Platform"]);
                    }
                    if (output.hasPlatformColor) {
                        imports.push(["RNPlatformColor", "PlatformColor"]);
                    }
                    if (imports.length > 0) {
                        (0, append_import_1.prependImports)(bodyNode, imports, "react-native");
                    }
                },
            },
        },
    };
}
exports.default = rootVisitor;
